@@ -18,6 +18,8 @@ import datetime
 import re
 import html
 
+from dateutil.tz import UTC
+
 from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import GObject
@@ -34,9 +36,13 @@ from .config import APP_NAME, APP_ID, APP_VERSION, FMT_DAY_LIST
 
 from . import models
 
+
 def _clean_markup(text):
     """remove unsupported markup from text to use in label"""
     text = text.replace("<p>", "").replace("</p>", "\n\n")
+    text = text.replace("<em>", "<i>").replace("</em>", "</i>")
+    text = text.replace("<code>", "<tt>").replace("</code>", "</tt>")
+    text = text.replace("<pre>", "\n<tt>").replace("</pre>", "</tt>\n")
     text = text.replace("<h1>", "<b>").replace("</h1>", "</b>")
     text = text.replace("<h2>", "<b>").replace("</h2>", "</b>")
     text = text.replace("<h3>", "<b>").replace("</h3>", "</b>")
@@ -68,9 +74,9 @@ class BasePage(Gtk.ScrolledWindow):
         self.column = Handy.Clamp(
             tightening_threshold=500, # dimensione sotto la quale decide di occupare tutto lo spazio
             maximum_size=640) # dimensione massima allragandosi
-        
+
         super().add(self.column)
-        
+
     def add(self, widget):
         self.column.add(widget)
 
@@ -82,16 +88,16 @@ class BaseListPage(BasePage):
     """Base class for ListBox based pages"""
     title = ""
     subtitle = ""
-    
+
     def __init__(self, title=None, subtitle=None, **kwargs):
         super().__init__(**kwargs)
-        
+
         title = title or self.title
         subtitle = subtitle or self.subtitle
 
         box = Gtk.VBox()
         box.get_style_context().add_class("main-box")
-        
+
         if title != "" or subtitle != "":
             titlebox = Gtk.VBox(margin_left=8, margin_right=8)
             if title != "":
@@ -106,41 +112,43 @@ class BaseListPage(BasePage):
                 titlebox.pack_start(label, False, False, 0)
 
             box.pack_start(titlebox, False, False, 2)
-            
-        
+
+
         frame = Gtk.Frame(valign=Gtk.Align.START)
-        
+
         listbox = Gtk.ListBox()
         listbox.connect('row-activated', self.on_activate)
         #listbox.get_style_context().add_class("content")
         listbox.set_header_func(self.build_list_header, None)
-        
+
         self.listbox = listbox
 
         # TODO: use list model?
         self.update_list()
-        
+
         frame.add(listbox)
         box.pack_start(frame, True, True, 2)
-        
+
         self.add(box)
-    
+
     def update_list(self, *args):
         self.data = list(self.get_objects())
         for w in self.listbox.get_children():
             w.destroy()
-            
+
         lastgroup = None
         for obj in self.data:
             mn = self.build_row(obj)
 
             self.listbox.add(mn)
         self.listbox.show_all()
-    
+
     def get_objects(self):
         return []
-    
+
     def build_list_header(self, row, before, *user_data):
+        if len(self.data) == 0:
+            return
         obj1 = self.data[row.get_index()]
         group_txt1 = self.group_by(obj1)
         if group_txt1 is None:
@@ -154,15 +162,15 @@ class BaseListPage(BasePage):
         if group_txt2 != group_txt1:
             h = ListRowHeader(group_txt1)
             row.set_header(h)
-    
+
     def group_by(self, obj):
         return None
-    
+
     def build_row(self, obj):
         row = Handy.ActionRow(activatable=True, selectable=False)
         row.set_title(str(obj))
         return row
-    
+
     def on_activate(self, listbox, actionrow):
         ...
 
@@ -177,7 +185,7 @@ class ConferencePage(BasePage):
 
     def __init__(self, conf, **kwargs):
         super().__init__(**kwargs)
-        
+
         meta = models.Meta()
 
         LBL_PROPS = {
@@ -187,15 +195,16 @@ class ConferencePage(BasePage):
         }
 
         box = Gtk.VBox(valign=Gtk.Align.CENTER)
-        
+
         self.logoimage = MaxSizedImage(max_width=200, max_height=200)
         f = conf.get_logo_file(cbk=self._set_logo)
         box.pack_start(self.logoimage, False, False, 16)
 
-        label = Gtk.Label(**LBL_PROPS)
-        label.set_markup("<big><b>{}</b></big>".format(_clean_markup(meta.title)))
-        box.pack_start(label, False, False, 16)
-        
+        if meta.title:
+            label = Gtk.Label(**LBL_PROPS)
+            label.set_markup("<big><b>{}</b></big>".format(_clean_markup(meta.title)))
+            box.pack_start(label, False, False, 16)
+
         if meta.venue is not None:
             label = Gtk.Label(html.unescape(meta.venue), **LBL_PROPS)
             label.get_style_context().add_class("dim-label")
@@ -214,7 +223,7 @@ class ConferencePage(BasePage):
             datebox.pack_start(label, False, False, 8)
         box.pack_start(datebox, False, False, 16)
 
-        
+
         links = conf.metadata.get('links',None)
         if links is not None:
             frame = Gtk.Frame()
@@ -224,7 +233,7 @@ class ConferencePage(BasePage):
                 if l['title'] != "Map":
                     row = WebLinkActionRow(html.unescape(l['title']), l['url'], l.get('type', None))
                     listbox.add(row)
-            
+
             frame.add(listbox)
             box.pack_start(frame, False, False, 16)
 
@@ -240,10 +249,10 @@ class ConferencePage(BasePage):
 class DaysPage(BaseListPage):
     """Days list page"""
     title = _("Days")
-    
+
     def get_objects(self):
         return models.Day.all()
-        
+
     def build_row(self, obj):
         row = Handy.ActionRow(activatable=True, selectable=False)
         row.set_title(obj.date.strftime(FMT_DAY_LIST))
@@ -262,10 +271,10 @@ class DaysPage(BaseListPage):
 class TracksPage(BaseListPage):
     """Tracks list page"""
     title = _("Tracks")
-    
+
     def get_objects(self):
         return models.Track.all()
-        
+
     #def group_by(self, obj):
     #    return obj.date.strftime(FMT_DAY_LIST)
 
@@ -281,7 +290,7 @@ class TracksPage(BaseListPage):
         subtitle = []
         if len(obj.room) == 1 and obj.room[0].name != "":
             subtitle.append(obj.room[0].name)
-        
+
         if len(obj.date) == 1:
             subtitle.append(obj.date[0].date.strftime(FMT_DAY_LIST))
 
@@ -296,7 +305,7 @@ class TracksPage(BaseListPage):
 class RoomsPage(BaseListPage):
     """Rooms list page"""
     title = _("Rooms")
-    
+
     def get_objects(self):
         return models.Room.all()
 
@@ -321,7 +330,7 @@ class EventsPage(BaseListPage):
         self.filters = filters
         self._group_by = group_by
         super().__init__(title=title, subtitle=subtitle)
-    
+
     def group_by(self, obj):
         if self._group_by == "day":
             return obj.date.strftime(FMT_DAY_LIST)
@@ -331,14 +340,14 @@ class EventsPage(BaseListPage):
             return obj.room
         else:
             return None
-    
+
     def get_objects(self):
         return models.Event.filter(**self.filters)
-    
+
     def build_row(self, obj):
         row = EventActionRow(obj)
         return row
-    
+
     def on_activate(self, listbox, actionrow):
         idx = actionrow.get_index()
         obj = self.data[idx]
@@ -346,6 +355,28 @@ class EventsPage(BaseListPage):
         self.pageStack.pushPage(page)
 
 
+class SearchEventsPage(EventsPage):
+    def __init__(self, searchbar):
+        self.searchbar = searchbar
+        self.nomatch_placeholder = PlaceholderWidget(
+            text = _("No matches found"),
+            icon_name = 'system-search'
+        )
+        self.nomatch_placeholder.show_all()
+        super().__init__()
+
+        self.listbox.set_placeholder(self.nomatch_placeholder)
+
+    def get_objects(self):
+        sb = self.searchbar
+        if sb.get_search_mode() and sb.entry.get_text() != "":
+            search = sb.entry.get_text().lower()
+            events = models.Event.search(search)
+        else:
+            events = []
+        return events
+
+
 class EventDetailPage(BasePage):
     """Talk details page"""
     def __init__(self, obj, **kwargs):
@@ -360,12 +391,12 @@ class EventDetailPage(BasePage):
 
         LBL_PROPS = {
             'justify': Gtk.Justification.FILL,
-            'halign' : Gtk.Align.START, 
+            'halign' : Gtk.Align.START,
             'wrap' : True,
         }
 
         box = Gtk.VBox( margin_left=8, margin_right=8)
-        
+
         titlebox = Gtk.VBox()
 
         label = Gtk.Label(**LBL_PROPS)
@@ -377,29 +408,29 @@ class EventDetailPage(BasePage):
             label.set_markup("{}".format(_clean_markup(obj.subtitle)))
             #label.get_style_context().add_class("dim-label")
             titlebox.pack_start(label, False, False, 0)
-            
+
         label = Gtk.Label(**LBL_PROPS)
         label.set_markup("<small>{}</small>".format(
             ", ".join([ _clean_markup(str(p)) for p in obj.persons()])))
         label.get_style_context().add_class("dim-label")
         titlebox.pack_start(label, False, False, 0)
-        
+
         box.pack_start(titlebox, False, False, 16)
 
         starbox = Gtk.HBox()
-        
+
         titlebox = Gtk.VBox()
         label = Gtk.Label(**LBL_PROPS)
         label.set_markup("<b>{}-{}</b>".format(
-            obj.start.strftime("%a %H:%M"),
-            obj.end.strftime("%H:%M")))
+            obj.start_in_tz().strftime("%a %H:%M"),
+            obj.end_in_tz().strftime("%H:%M")))
         titlebox.pack_start(label, False, False, 0)
 
         if obj.room:
             label = Gtk.Label(**LBL_PROPS)
             label.set_markup(_("Room: <b>{}</b>").format(obj.room))
             titlebox.pack_start(label, False, False, 0)
-        
+
         if obj.track:
             label = Gtk.Label(**LBL_PROPS)
             label.set_markup(_("Track: <b>{}</b>").format(obj.track))
@@ -407,11 +438,11 @@ class EventDetailPage(BasePage):
 
         starbox.pack_start(titlebox, False, True, 0)
 
-        icon_name = "starred-symbolic" if obj.starred else "non-starred-symbolic" 
+        icon_name = "starred-symbolic" if obj.starred else "non-starred-symbolic"
         self.toggle_btn = Gtk.Button.new_from_icon_name(icon_name, Gtk.IconSize.BUTTON)
         self.toggle_btn.connect("clicked", self.toggle_starred)
         starbox.pack_end(self.toggle_btn, False, False, 16)
-        
+
         box.pack_start(starbox, False, False, 16)
 
         self.conflicts = list(obj.get_conflicts())
@@ -458,13 +489,42 @@ class EventDetailPage(BasePage):
             for l in obj.links():
                 row = WebLinkActionRow(l.name, l.href)
                 listbox.add(row)
-            
+
             frame.add(listbox)
             linkbox.pack_start(frame, False, False, 0)
             box.pack_start(linkbox, False, False, 16)
 
         self.add(box)
-    
+
+        # progress indicator
+        Gio.Application.get_default().connect(
+            "tick", lambda *_: self.queue_draw()
+        )
+
+        box.connect("draw", self._on_draw)
+
+    def _on_draw(self, widget, cr):
+        now = datetime.datetime.now(UTC).replace(tzinfo=None)
+        obj = self.obj
+
+        w = self.get_allocated_width()
+
+        dur = (obj.end.timestamp() - obj.start.timestamp())
+        pos = (now.timestamp() - obj.start.timestamp())
+        if dur > 0:
+            prc = min(max(pos/dur, 0.0), 1.0)
+        else:
+            # not a valid event duration
+            # set prc to 0 if the event is in the future or 1 if in the past
+            prc = int(pos >= 0)
+
+        x = w * prc
+
+        cr.set_source_rgb(0, 0, 0)
+        cr.move_to(0, 1)
+        cr.line_to(x, 1)
+        cr.stroke()
+
     def toggle_starred(self, button):
         self.obj.set_star(not self.obj.starred)
         icon_name = "starred-symbolic" if self.obj.starred else "non-starred-symbolic"
@@ -479,21 +539,41 @@ class EventDetailPage(BasePage):
 
 class StarredPage(BaseListPage):
     """Starred list page. This page is not in leaflet, but in mainstack"""
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        placeholder = PlaceholderWidget(
+    def __init__(self, searchbar, **kwargs):
+        self.empty_placeholder = PlaceholderWidget(
             _("Starred Talks"),
             _("You don't have any starred talk yet.") + "\n" + \
             _("Click on the star button to add a talk to this list " + \
                 "and to be reminded when they are about to start."),
             "starred")
-        placeholder.show_all()
+        self.empty_placeholder.show_all()
+        self.nomatch_placeholder = PlaceholderWidget(
+            text = _("No matches found"),
+            icon_name = 'system-search'
+        )
+        self.nomatch_placeholder.show_all()
+
+        self.searchbar = searchbar
+
+        super().__init__(**kwargs)
+
         self.get_style_context().add_class("starred-page")
         self.get_style_context().add_class("clean-list")
-        self.listbox.set_placeholder(placeholder)
-    
+        self.listbox.set_placeholder(self.empty_placeholder)
+
     def get_objects(self):
         return models.Event.filter(starred = True)
+
+    def get_objects(self):
+        sb = self.searchbar
+        if sb.get_search_mode() and sb.entry.get_text() != "":
+            self.listbox.set_placeholder(self.nomatch_placeholder)
+            search = sb.entry.get_text().lower()
+            events = models.Event.search(search, starred = True)
+        else:
+            self.listbox.set_placeholder(self.empty_placeholder)
+            events = models.Event.filter(starred = True)
+        return events
     
     def build_row(self, obj):
         row = EventActionRow(obj)
@@ -503,18 +583,18 @@ class StarredPage(BaseListPage):
         idx = actionrow.get_index()
         obj = self.data[idx]
         page = EventDetailPage(obj=obj)
-        self.pageStack.replaceWithPage(page)
+        self.pageStack.pushPage(page)
 
 
 class MainMenuPage(Gtk.ScrolledWindow):
     """Leaflet sidebar menu"""
-    
+
     ITEMS = (
         ["EVENT", "emblem-system-symbolic", ConferencePage, None],
         (_("Days"), "x-office-calendar-symbolic", DaysPage, models.Day.count),
         (_("Tracks"), "format-justify-fill-symbolic", TracksPage, models.Track.count),
         (_("Rooms"),"display-projector-symbolic", RoomsPage, models.Room.count))
-    
+
     def __init__(self, pageStack, conf):
         super().__init__(
             width_request=270,
@@ -524,9 +604,9 @@ class MainMenuPage(Gtk.ScrolledWindow):
 
         self.pageStack = pageStack
         self.conf = conf
-        
+
         self.ITEMS[0][0] = conf.title
-        
+
         self.listbox = Gtk.ListBox()
         self.update()
         self.listbox.connect('row-activated', self.on_activate)
@@ -544,6 +624,12 @@ class MainMenuPage(Gtk.ScrolledWindow):
             self.listbox.add(mn)
             self.listbox.show_all()
 
+    def select_conference_page(self):
+        """select and show ConferencePAge"""
+        row = self.listbox.get_row_at_index(0)
+        self.listbox.select_row(row)
+        self.on_activate(self.listbox, row)
+
     def on_activate(self, listbox, actionrow):
         idx = actionrow.item_id
         cls = self.ITEMS[idx][2]
