@@ -18,6 +18,8 @@ from os import path
 import datetime
 import html
 
+from dateutil.tz import UTC
+
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gio
@@ -59,7 +61,8 @@ class WebLinkActionRow(Handy.ActionRow):
             icontype = ftype
         if fmin == 'pdf' or 'document' in fmin:
             icontype = ftype
-            
+
+        self.content_type = icontype
         ficon = Gio.content_type_get_icon(icontype)
 
         # I have to manage icon by hand.. ouf
@@ -68,18 +71,22 @@ class WebLinkActionRow(Handy.ActionRow):
 
 
 def weblinkactionrowactivated(listbox, actionrow):
-    Gio.app_info_launch_default_for_uri(actionrow.url, None)
+    appinfo = Gio.app_info_get_default_for_type(actionrow.content_type, True)
+    if appinfo is not None:
+        appinfo.launch_uris([actionrow.url], None)
+    else:
+        Gio.app_info_launch_default_for_uri(actionrow.url, None)
+
 
 
 
-        
 class EventActionRow(Handy.ActionRow):
     def __init__(self, obj):
         super().__init__(activatable=True, selectable=False)
-        
+
         subtitle = "{}-{}".format(
-            obj.start.strftime("%a %H:%M"),
-            obj.end.strftime("%H:%M"))
+            obj.start_in_tz().strftime("%a %H:%M"),
+            obj.end_in_tz().strftime("%H:%M"))
 
         if obj.room:
             subtitle = "{} ({})".format(
@@ -96,22 +103,50 @@ class EventActionRow(Handy.ActionRow):
             subtitle = "{} - {}".format(
                 subtitle,
                 ", ".join([str(p) for p in ps]))
-        
+
         self.set_title(html.unescape(obj.title))
         self.set_subtitle(html.unescape(subtitle))
         self.obj = obj
 
         obj.connect('update', self.on_update)
-        
-        icon_name = "starred-symbolic" if obj.starred else "non-starred-symbolic" 
+
+        icon_name = "starred-symbolic" if obj.starred else "non-starred-symbolic"
         self.toggle_btn = Gtk.Button.new_from_icon_name(icon_name, Gtk.IconSize.BUTTON)
         self.toggle_btn.connect("clicked", self.toggle_starred)
-        self.add_prefix(self.toggle_btn)
-   
+        self.add(self.toggle_btn)
+
+        Gio.Application.get_default().connect(
+            "tick", lambda *_: self.queue_draw()
+        )
+
+        self.connect("draw", self._on_draw)
+
+    def _on_draw(self, widget, cr):
+        now = datetime.datetime.now(UTC).replace(tzinfo=None)
+        obj = self.obj
+
+        h = self.get_allocated_height()
+
+        dur = (obj.end.timestamp() - obj.start.timestamp())
+        pos = (now.timestamp() - obj.start.timestamp())
+        if dur > 0:
+            prc = min(max(pos/dur, 0.0), 1.0)
+        else:
+            # not a valid event duration
+            # set prc to 0 if the event is in the future or 1 if in the past
+            prc = int(pos >= 0)
+
+        y = h * prc
+
+        cr.set_source_rgb(0, 0, 0)
+        cr.move_to(1, 0)
+        cr.line_to(1, y)
+        cr.stroke()
+
     def on_update(self, *args):
         icon_name = "starred-symbolic" if self.obj.starred else "non-starred-symbolic"
         self.toggle_btn.get_image().set_from_icon_name(icon_name, Gtk.IconSize.BUTTON)
-            
+
     def toggle_starred(self, button):
         self.obj.set_star(not self.obj.starred)
 
@@ -132,7 +167,7 @@ class PlaceholderWidget(Gtk.VBox):
         super().__init__(valign = Gtk.Align.CENTER)
         img = Gtk.Image.new_from_pixbuf(
             Gtk.IconTheme.get_default().load_icon(
-                icon_name, 256, 0))
+                icon_name, 128, 0))
         self.pack_start(img, False, False, 16)
 
         LBL_PROPS = {
@@ -194,7 +229,7 @@ class ErrorWidget(Gtk.VBox):
         lbl1.set_markup("<b>{}</b>".format(msg))
         lbl2 = Gtk.Label(str(error))
         lbl2.get_style_context().add_class("dim-label")
-        
+
         self.pack_start(img, False, False, 16)
         self.pack_start(lbl1, False, False, 0)
         self.pack_start(lbl2, False, False, 0)
@@ -260,6 +295,28 @@ class LoadingOverlay(Gtk.Revealer):
         self.set_reveal_child(False)
 
 
+class SearchBarOverlay(Handy.SearchBar):
+    def __init__(self, **kwargs):
+        opts = dict(
+            halign = Gtk.Align.FILL,
+            valign = Gtk.Align.START,
+            show_close_button = True
+        )
+        opts.update(kwargs)
+        super().__init__(**opts)
+        clamp  = Handy.Clamp()
+        self.entry = Gtk.SearchEntry(can_focus=True)
+        clamp.add(self.entry)
+        self.add(clamp)
+        self.show_all()
+
+        self.connect('notify::search-mode-enabled', self._on_search_mode_changed)
+
+    def _on_search_mode_changed(self, *args):
+        if self.get_search_mode():
+            self.entry.set_text("")
+            self.entry.grab_focus()
+
 class OkCancelDialog(Gtk.Dialog):
     CANCEL = 0
     OK = 1
@@ -540,6 +597,7 @@ class SimpleScalableImage(Gtk.Image):
         self._originalpb = pixbuf
         super().set_from_pixbuf(pixbuf)
 
+
 class MaxSizedImage(Gtk.Image):
     def __init__(self, max_width, max_height, **kwargs):
         super().__init__(**kwargs)
@@ -565,3 +623,4 @@ class MaxSizedImage(Gtk.Image):
                 h,
                 GdkPixbuf.InterpType.BILINEAR)
         super().set_from_pixbuf(pixbuf)
+
