@@ -16,6 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import datetime
+import re
 
 from gi.repository import GLib
 from gi.repository import Gtk
@@ -31,7 +32,7 @@ from . import local
 from . import models
 from .preferences import PreferencesWindow
 from .widgets import LoadingWidget, ErrorWidget, PlaceholderWidget
-from .widgets import NotificationOverlay, LoadingOverlay
+from .widgets import NotificationOverlay, LoadingOverlay, SearchBarOverlay
 from .widgets import InputDialog, ConferenceEditDialog
 
 
@@ -58,11 +59,45 @@ class NetworkConnectionMonitor(GObject.Object):
 
 
 class StartView(pages.BaseListPage):
+    """
+    window
+        overlay
+            notification
+            loading
+        title_bar
+            open_button
+            search_button
+            update_button
+
+
+        box
+            search
+            column
+                box <VBox>
+                    titlebox <VBox> (if title or subtitle is set)
+                        title       <Label>
+                        subtitle    <Label>
+                frame
+                    listbox
+    """
+
     def __init__(self, window, **kwargs):
         self.window = window
         self.nm = NetworkConnectionMonitor()
         self.data = []
 
+        self.empty_placeholder = PlaceholderWidget(
+            APP_NAME,
+            _("No conferences yet.") + "\n" + \
+            _("Add a conference with the add button in title bar."))
+        self.empty_placeholder.show_all()
+
+        self.nomatch_placeholder = PlaceholderWidget(
+            text = _("No matches found"),
+            icon_name = 'system-search'
+        )
+        self.nomatch_placeholder.show_all()
+
         super().__init__(**kwargs)
 
         self.get_style_context().add_class("start-view")
@@ -76,11 +111,7 @@ class StartView(pages.BaseListPage):
             title = APP_NAME
         )
 
-        placeholder = PlaceholderWidget(
-            APP_NAME,
-            _("No conferences yet.") + "\n" + \
-            _("Add a conference with the add button in title bar."))
-        self.listbox.set_placeholder(placeholder)
+        self.listbox.set_placeholder(self.empty_placeholder)
 
         fetcher = local.open_menu(self.nm.props.isconnected)
         if fetcher is not None:
@@ -92,8 +123,7 @@ class StartView(pages.BaseListPage):
 
         # app menu
         appmenumodel = Gio.Menu()
-        appmenumodel.append(_("Preferences"), "win.show-preferences")
-        appmenumodel.append(_("About {}").format(APP_NAME), "win.about")
+        self.window.pop_menu(appmenumodel)
 
         self.start_button = Gtk.Button.new_from_icon_name('open-menu-symbolic', 1)
         self.start_button.set_tooltip_text(_("Show window menu"))
@@ -113,6 +143,21 @@ class StartView(pages.BaseListPage):
         self.open_button.connect('clicked', self.open_custom_url)
         title_bar.pack_start(self.open_button)
 
+        _search_button_icon = Gtk.Image.new_from_pixbuf(
+            Gtk.IconTheme.get_default().load_icon(
+                'system-search-symbolic', 16, 0))
+        self.search_button = Gtk.ToggleButton.new()
+        self.search_button.set_image(_search_button_icon)
+        self.search_button.set_tooltip_text(_("Find event"))
+        self.search_button.bind_property("active",
+            self.window.searchbar, "search-mode-enabled",
+            GObject.BindingFlags.BIDIRECTIONAL | GObject.BindingFlags.SYNC_CREATE )
+        title_bar.pack_start(self.search_button)
+
+        # search event
+        self.window.searchbar.entry.connect('notify::text', self.on_do_search)
+        self.window.searchbar.connect('notify::search-mode-enabled', self.on_do_search)
+
         # can't do this because: Warning: ../glib/gobject/gbinding.c:271: Unable to convert a value of type PyObject to a value of type gboolean
         #self.nm.bind_property("isconnected", update_action, "enabled", 0 )
         self.nm.connect("notify",self.on_networkconnectionmonitor_notify)
@@ -121,11 +166,14 @@ class StartView(pages.BaseListPage):
         title_bar.show_all()
         self.show_all()
 
+    def on_do_search(self, *args):
+        self.update_list()
+
     def on_networkconnectionmonitor_notify(self, *args):
         is_connected = self.nm.props.isconnected
         self.update_button.set_sensitive(is_connected)
         self.open_button.set_sensitive(is_connected)
-        
+
     def update_schedules(self, *args):
         fetcher = local.update_menu()
         def _done(s, f):
@@ -138,7 +186,7 @@ class StartView(pages.BaseListPage):
         self.window.loading.show(fetcher)
         fetcher.connect("done", _done)
         fetcher.connect("error", _error)
-    
+
     def edit_meta(self, url):
         conf = models.Conference.by_url(url)
         if conf is None:
@@ -188,8 +236,19 @@ class StartView(pages.BaseListPage):
 
     def get_objects(self):
         confs = self.data
+        sb = self.window.searchbar
         try:
             confs = models.Conference.all()
+
+            # if searchbar is visible and search text is not "", filter confs by title and url
+            if (sb.get_search_mode() and sb.entry.get_text() != ""):
+                self.listbox.set_placeholder(self.nomatch_placeholder)
+                search = sb.entry.get_text().lower()
+                confs = [ conf for conf in confs
+                            if search in conf.title.lower() or search in conf.url.lower() ]
+            else:
+                self.listbox.set_placeholder(self.empty_placeholder)
+
         except Exception as e:
             self.window.notification.show(_("<b>Error loading events list:</b>\n{}").format(e))
         return confs
@@ -200,7 +259,7 @@ class StartView(pages.BaseListPage):
             return _("Past")
         else:
             return _("Coming up")
-        
+
     def build_row(self, obj):
         row = Handy.ActionRow(activatable=True, selectable=False)
         row.set_title(obj.title)
@@ -215,27 +274,39 @@ class StartView(pages.BaseListPage):
 class MainView(Gtk.VBox):
     """
     window
-        overlay: notification
+        overlay
+            notification
+            loading
         title_bar
-            start_button
-            update_button
+            back_button
+            search_button
             'squeezer'
                 top_switcher
                 title_label
-        box (self)
-            mainstack
-                leaflet ("schedule")
-                    main_menu
-                    Gtk.Separator
-                    stack <Handy.Deck>
+            update_button
+            start_button
+        box
+            search
+            box (self)
+                mainstack
+                    leaflet ("schedule")
+                        main_menu
+                        Gtk.Separator
+                        stack_schedule <Handy.Deck>
+                            [pages]
+                    stack_starred ("starred") <Handy.Deck>
+                        starred
                         [pages]
-                starred ("starred")
-                map ("map")
-            bottom_switcher
+
+                    map ("map")
+                bottom_switcher
+
+    TODO: this view is handling too much things!
     """
+
     def __init__(self, window, conf):
         super().__init__()
-        
+
         title_bar = Handy.HeaderBar(
             centering_policy = Handy.CenteringPolicy.STRICT,
             show_close_button = True,
@@ -255,8 +326,7 @@ class MainView(Gtk.VBox):
         if conf.user:
             appmenumodel.append(_("Edit info"), "win.edit-meta")
         appmenumodel.append(_("Return to events list"), "win.return-to-events")
-        appmenumodel.append(_("Preferences"), "win.show-preferences")
-        appmenumodel.append(_("About {}").format(APP_NAME), "win.about")
+        self.window.pop_menu(appmenumodel)
 
         self.start_button = Gtk.Button.new_from_icon_name('open-menu-symbolic', 1)
         self.start_button.set_tooltip_text(_("Show window menu"))
@@ -299,18 +369,35 @@ class MainView(Gtk.VBox):
 
         self.nm.connect("notify",self.on_networkconnectionmonitor_notify)
 
+        # back button
         self.back_button = Gtk.Button.new_from_icon_name(
                 'go-previous-symbolic', 1)
+        self.back_button.set_tooltip_text(_("Go back"))
         self.back_button.connect('clicked', self.goBack)
         title_bar.pack_start(self.back_button)
-        
+
+        # search button
+        _search_button_icon = Gtk.Image.new_from_pixbuf(
+            Gtk.IconTheme.get_default().load_icon(
+                'system-search-symbolic', 16, 0))
+        self.search_button = Gtk.ToggleButton.new()
+        self.search_button.set_image(_search_button_icon)
+        self.search_button.set_tooltip_text(_("Find talk"))
+        self.search_button.bind_property("active",
+            self.window.searchbar, "search-mode-enabled",
+            GObject.BindingFlags.BIDIRECTIONAL | GObject.BindingFlags.SYNC_CREATE )
+        title_bar.pack_start(self.search_button)
+
+        # search event
+        self.window.searchbar.entry.connect('notify::text', self.on_do_search)
+        self.window.searchbar.connect('notify::search-mode-enabled', self.on_search_mode_changed)
+
         # mainstack
-        
         self.mainstack = Gtk.Stack(
             transition_type=Gtk.StackTransitionType.NONE,
             vexpand_set=True,
             vexpand=True)
-        
+
         self.leaflet = Handy.Leaflet(
             can_swipe_back=True,
             hhomogeneous_folded=True,
@@ -320,34 +407,49 @@ class MainView(Gtk.VBox):
         self.leaflet.connect('notify::visible-child', self.on_leaflet_notify)
         self.main_menu = pages.MainMenuPage(pageStack=self, conf=self.conf)
 
-        self.stack_count = 1
-        self.stack = Handy.Deck(
+        self.stack_schedule = Handy.Deck(
             can_swipe_back=True,
             can_swipe_forward=False)
-        #self.stack.connect("notify::visible-child", self.on_stack_changed)
-        self.stack.connect("notify::transition-running", self.on_stack_changed)
-        
+        self.stack_schedule.connect("notify::transition-running", self.on_stack_changed)
+
         # add main menu and stack to leaflet
         self.leaflet.add(self.main_menu)
         sep = Gtk.Separator()
         self.leaflet.add(sep)
         self.leaflet.child_set_property(sep, 'navigatable', False)
-        self.leaflet.add(self.stack)
-        
-        # add first page to stack
-        self.stack.add(pages.ConferencePage(self.conf))
-        
+        self.leaflet.add(self.stack_schedule)
+
+        # add first page to stack_schedule
+        self.stack_schedule.add(pages.ConferencePage(self.conf))
+        self.stack_schedule.count = 1
+
+
+        # starred
+        self.stack_starred = Handy.Deck(
+            can_swipe_back=True,
+            can_swipe_forward=False)
+        self.stack_starred.connect("notify::transition-running", self.on_stack_changed)
+
+        self.starred = pages.StarredPage(searchbar=self.window.searchbar)
+        self.starred._set_pagestack(self)
+
+        # add first page to stack_starred
+        self.stack_starred.add(self.starred)
+        self.stack_starred.count = 1
+
+        # set current stack
+        self.stack = self.stack_schedule
+
 
         # main stack content
 
         self.mainstack.add_titled(self.leaflet, "schedule", _("Schedule"))
         self.mainstack.child_set_property(self.leaflet, "icon-name", "x-office-calendar-symbolic")
-        
-        self.starred = pages.StarredPage()
-        self.starred._set_pagestack(self)
-        self.mainstack.add_titled(self.starred, "starred", _("Starred"))
-        self.mainstack.child_set_property(self.starred, "icon-name", "starred-symbolic")
-        
+
+
+        self.mainstack.add_titled(self.stack_starred, "starred", _("Starred"))
+        self.mainstack.child_set_property(self.stack_starred, "icon-name", "starred-symbolic")
+
 
         if len(self.conf.get_map_links()) > 0:
             self.map = pages.MapPage(self.conf)
@@ -360,10 +462,10 @@ class MainView(Gtk.VBox):
             stack=self.mainstack)
         self.bottom_switcher = Handy.ViewSwitcherBar(
             stack=self.mainstack)
-        
+
         #mainstack event
         self.mainstack.connect("notify::visible-child", self.on_mainstack_changed)
-        
+
         squeezer = Handy.Squeezer()
         squeezer.add(self.top_switcher)
         title_label = Gtk.Label()
@@ -371,21 +473,22 @@ class MainView(Gtk.VBox):
         squeezer.add(title_label)
         title_bar.set_custom_title(squeezer)
         squeezer.connect("notify::visible-child",self.on_headerbar_squeezer_notify)
-        
+
         # main box contanier
 
         self.pack_start(self.mainstack, True, True, 0)
         self.pack_start(self.bottom_switcher, False, False, 0)
-        
+
         title_bar.show_all()
         self.show_all()
-        
-        # event notification timer
-        # every minute
-        GLib.timeout_add_seconds(60, self.check_event_notification)
+
+        # Check if there are notifications to send
+        Gio.Application.get_default().connect(
+            'tick', self.check_event_notification
+        )
         self.check_event_notification()
 
-        self.update_back_button()
+        self.update_titlebar_buttons()
 
         # set start mainstack page
         self.mainstack.set_visible_child(self.leaflet)
@@ -395,16 +498,29 @@ class MainView(Gtk.VBox):
         self.update_button.set_sensitive(is_connected)
 
     def on_mainstack_changed(self, *args):
-        self.update_back_button()
-        if (self.mainstack.get_visible_child() == self.starred):
+        if (self.mainstack.get_visible_child() == self.stack_starred):
+            self.stack = self.stack_starred
+            # get back to starred list when switching main stack
+            # would be better not, but as we can't update EventDetailPage
+            # without rebuild it, if use is in a starred EventDetailPage
+            # then get back to schedule and change something related to that
+            # event, EventDetailPage objects in stack_starred will be out
+            # of sync (eg, a new starred event which overlaps)
             self.starred.update_list()
-        
+            self.stack.set_visible_child(self.starred)
+            self.stack.count = 1
+            for w in self.stack.get_children()[1:]:
+                w.destroy()
+        elif (self.mainstack.get_visible_child() == self.leaflet):
+            self.stack = self.stack_schedule
+        self.update_titlebar_buttons()
+
     def on_headerbar_squeezer_notify(self, squeezer, event):
         child = squeezer.get_visible_child()
         self.bottom_switcher.set_reveal(child != self.top_switcher)
-    
+
     def on_leaflet_notify(self, sender, prop):
-        self.update_back_button()
+        self.update_titlebar_buttons()
 
     def on_fetcher_done(self, *args):
         self.foreach(lambda obj: obj.destroy())
@@ -416,6 +532,40 @@ class MainView(Gtk.VBox):
         self.pack_start(e, True, True, 0)
         e.show_all()
 
+    def on_do_search(self, *args):
+        if self.mainstack.get_visible_child_name() == "schedule":
+            searchpage = None
+            if self.stack.count > 0:
+                searchpage = self.stack.get_children()[0]
+
+            if searchpage.__class__ != pages.SearchEventsPage:
+                searchpage = pages.SearchEventsPage(searchbar = self.window.searchbar)
+                self.replaceWithPage(searchpage)
+
+            if self.stack.count > 1:
+                self.stack.set_visible_child(searchpage)
+                self.stack.count = 1
+                for w in self.stack.get_children()[1:]:
+                    w.destroy()
+            searchpage.update_list()
+
+        elif self.mainstack.get_visible_child_name() == "starred":
+            if self.stack.count > 1:
+                # reset stack to first page
+                self.stack.set_visible_child(self.starred)
+                self.stack.count = 1
+                for w in self.stack.get_children()[1:]:
+                    w.destroy()
+            self.starred.update_list()
+
+    def on_search_mode_changed(self, *args):
+        if self.mainstack.get_visible_child_name() == "schedule":
+            searchmode = self.window.searchbar.get_search_mode()
+            currentfirstpage = self.stack.get_children()[0]
+            if not searchmode and currentfirstpage.__class__ == pages.SearchEventsPage:
+                # search closed, reset to ConferencePage
+                self.main_menu.select_conference_page()
+
     def edit_meta(self):
         d = ConferenceEditDialog(
             self.window, self.conf, can_cancel=True, can_delete=True)
@@ -440,7 +590,8 @@ class MainView(Gtk.VBox):
     def _update_ui(self):
         """update ui when database change. Reset view to ConferencePage"""
         self.main_menu.update()
-        self.replaceWithPage(pages.ConferencePage(self.conf))
+        self.mainstack.set_visible_child_name("schedule")
+        self.main_menu.select_conference_page()
 
     def update_event(self, *args):
         """update cached event data"""
@@ -460,17 +611,20 @@ class MainView(Gtk.VBox):
         #TODO UPDATE GUI!!! ! !
 
     def check_event_notification(self, *args):
-        nextup = models.Event.nextup(5)
+        nextup = list(models.Event.nextup(5))
         for e in nextup:
+            # fast and stupid clean html
+            cleanr = re.compile('<.*?>|&([a-z0-9]+|#[0-9]{1,6}|#x[0-9a-f]{1,6});')
+            title = re.sub(cleanr, '', e.title)
+            print("nextup:", e.start_in_tz().strftime("%H:%M"), title, "@", e.room)
             self._send_desktop_notification(
-                _("Next up: {}").format(e.title),
+                _("Next up: {}").format(title),
                 _("at {} in {}").format(
-                    e.start.strftime("%H:%M"),
+                    e.start_in_tz().strftime("%H:%M"),
                     e.room
                 ),
                 f"nextup-{e.id}")
             e.set_notified(True)
-        return True # keep timer running
 
     def _send_desktop_notification(self, title, body, nid=None):
         if nid in self.notifications:
@@ -492,15 +646,18 @@ class MainView(Gtk.VBox):
         Gio.Application.get_default().send_notification(nid, n)
     """
 
-    def update_back_button(self):
-        inleaflet = self.mainstack.get_visible_child() == self.leaflet
-        if inleaflet:
+    def update_titlebar_buttons(self):
+        search_show = True
+        back_show = False
+        if self.mainstack.get_visible_child_name() == "schedule":
             folded = self.leaflet.get_property('folded')
             firstfold = self.leaflet.get_visible_child() == self.main_menu
-            show = (folded and not firstfold) or self.stack_count > 1
-        else:
-            show = False
-        self.back_button.set_visible(show)
+            back_show = (folded and not firstfold) or self.stack.count > 1
+        elif self.mainstack.get_visible_child_name() == "starred":
+            back_show = self.stack.count > 1
+
+        self.back_button.set_visible(back_show)
+        self.search_button.set_visible(search_show)
 
     def on_stack_changed(self, *args):
         if self.stack.get_transition_running():
@@ -512,19 +669,19 @@ class MainView(Gtk.VBox):
         if currentid == (lastid-1):
             # we got back one page, destroy last page
             stack_children[lastid].destroy()
-            self.stack_count = self.stack_count - 1
-        self.update_back_button()
+            self.stack.count = self.stack.count - 1
+        self.update_titlebar_buttons()
 
     def goBack(self, *args):
-        if (self.stack_count > 1):
+        if (self.stack.count > 1):
             self.stack.navigate(Handy.NavigationDirection.BACK)
             """
             #self.stack.set_transition_type(Gtk.StackTransitionType.SLIDE_RIGHT)
             current = self.stack.get_children()[-1]
             previous = self.stack.get_children()[-2]
             self.stack.set_visible_child(previous)
-            self.stack_count = self.stack_count - 1
-            
+            self.stack.count = self.stack.count - 1
+
             folded = self.leaflet.get_property('folded')
             while not folded and self.leaflet.get_child_transition_running():
                 pass
@@ -532,39 +689,49 @@ class MainView(Gtk.VBox):
             """
         else:
             self.leaflet.set_visible_child(self.main_menu)
-        self.update_back_button()
+        self.update_titlebar_buttons()
 
     def pushPage(self, page):
-        self.mainstack.set_visible_child(self.leaflet)
+        #self.mainstack.set_visible_child(self.leaflet)
         page._set_pagestack(self)
         self.stack.add(page)
-        self.stack_count = self.stack_count + 1
+        self.stack.count = self.stack.count + 1
         page.show_all()
         self.leaflet.set_visible_child(self.stack)
         self.stack.set_visible_child(page)
 
-        self.update_back_button()
+        self.update_titlebar_buttons()
 
     def replaceWithPage(self, page):
-        self.mainstack.set_visible_child(self.leaflet)
+        #self.mainstack.set_visible_child(self.leaflet)
         page._set_pagestack(self)
         self.stack.add(page)
         page.show_all()
         self.leaflet.set_visible_child(self.stack)
         self.stack.set_visible_child(page)
-        self.stack_count = 1
-        
+        self.stack.count = 1
+
         #folded = self.leaflet.get_property('folded')
         #while not folded and self.leaflet.get_child_transition_running():
         #    pass
-        
+
         for w in self.stack.get_children()[:-1]:
             w.destroy()
 
-        self.update_back_button()
+        self.update_titlebar_buttons()
 
 
 class ConfyWindow(Gtk.ApplicationWindow):
+    """
+        ConfyWindow
+            overlay
+                notification    <NotificationOverlay>
+                loading         <LoadingOverlay>
+                box             <VBox>
+                    searchbar       <SearchBarOverlay>
+                    view            <MainView>|<StartView>
+    """
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
 
@@ -580,6 +747,12 @@ class ConfyWindow(Gtk.ApplicationWindow):
         self.loading = LoadingOverlay()
         self.overlay.add_overlay(self.loading)
 
+        self.box = Gtk.VBox()
+        self.searchbar = SearchBarOverlay()
+        self.box.pack_start(self.searchbar, False, False, 0)
+        self.overlay.add(self.box)
+
+
         # win actions
         edit_meta_action = Gio.SimpleAction.new("edit-meta", None)
         edit_meta_action.connect("activate", self.edit_meta)
@@ -593,13 +766,17 @@ class ConfyWindow(Gtk.ApplicationWindow):
         show_preferences_action.connect("activate", self.show_preferences )
         self.add_action(show_preferences_action)
 
+        search_action = Gio.SimpleAction.new("search", None)
+        search_action.connect("activate", lambda *_: self.searchbar.set_search_mode(True) )
+        self.add_action(search_action)
+
         about_action = Gio.SimpleAction.new("about", None)
         about_action.connect("activate", self.show_about)
         self.add_action(about_action)
 
         self.view = None
         self.show_all()
-        
+
         self.show_start()
 
     def show_preferences(self, *args):
@@ -610,6 +787,13 @@ class ConfyWindow(Gtk.ApplicationWindow):
         if isinstance(self.view, MainView):
             self.view.edit_meta()
 
+    def pop_menu(self, menu):
+        s = Gio.Menu()
+        s.append(_("Preferences"), "win.show-preferences")
+        s.append(_("Keyboard shortcuts"), "app.shortcuts")
+        s.append(_("About {}").format(APP_NAME), "win.about")
+        menu.append_section(None, s)
+
     def show_about(self, *args):
         Gtk.AboutDialog(
             program_name=APP_NAME,
@@ -617,7 +801,7 @@ class ConfyWindow(Gtk.ApplicationWindow):
             version=APP_VERSION,
             copyright="© 2020 Fabio Comuni",
             website="https://confy.kirgroup.net/",
-            authors=["Fabio Comuni"],
+            authors=["Fabio Comuni", "Evangelos Ribeiro Tzaras", "Sebastian Crane"],
             comments=_("Conference schedules viewer"),
             license_type=Gtk.License.GPL_3_0
         ).show()
@@ -626,15 +810,17 @@ class ConfyWindow(Gtk.ApplicationWindow):
         """start page, list conferences"""
         if self.view is not None:
             self.view.destroy()
+        self.searchbar.set_search_mode(False)
         self.view = StartView(self)
-        self.overlay.add(self.view)
+        self.box.pack_start(self.view, True, True, 0)
         local.close()
-    
+
     def show_main(self, conf):
         """start page, list conferences"""
         if self.view is not None:
             self.view.destroy()
+        self.searchbar.set_search_mode(False)
         self.view = MainView(self, conf)
-        self.overlay.add(self.view)
+        self.box.pack_start(self.view, True, True, 0)
 
 
